#!/usr/bin/perl
# Copyright Jean-Philippe Guillemin <jp.guillemin@free.fr>. This program is free software; you can redistribute
# it and/or modify it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at your option)
# any later version. Please take a look at http://www.gnu.org/copyleft/gpl.htm
#
# Netpkg is a tool for easily install or upgrade packages via the network. With Netpkg,
# you can make a minimal installation of Zenwalk Linux and install/upgrade just the
# packages you need most.
#
#


use strict;
use File::Path;
use Getopt::Std;
use POSIX;
use IO::Scalar;
use File::Find ();
use ZW::Netpkg::Netpkg;
use ZW::Netpkg::PVfilter;
use HTML::Entities qw(encode_entities_numeric);
use Locale::gettext;


use Gtk2 '-init' ;
use Glib; 
set_locale Gtk2;  # internationalize


my $version="4.1";

#set debug to 1 to enable
my $debug = 0;


####################################################################
############################ CONSTANTS  ############################
####################################################################

use constant CATEGORY => 0;
use constant AVAILABLE_PKG => 1;
use constant SORTSTRING => 2;
use constant SIZE_COMPRESSED => 3;
use constant SIZE_UNCOMPRESSED => 4;
use constant DEPENDENCIES => 5;
use constant AVAILABLE_VERSION => 6;
use constant AVAILABLE_BUILD => 7;
use constant DESCRIPTION => 8;
use constant INSTALLED_PKG => 9;
use constant INSTALLED_VERSION => 10;
use constant INSTALLED_BUILD => 11;
use constant STATUS => 12;
use constant SELECTED_AS_PKG => 13;
use constant SELECTED_AS_DEP => 14;
use constant BLACKLISTED => 15;
use constant PATH => 16;

use constant STATUS_OK => 0;
use constant STATUS_DOWNGRADED => -1;
use constant STATUS_UPDATED => 1;
use constant STATUS_NEW => 2;
use constant STATUS_UNAVAILABLE => -2;

use constant NONE => 0b00000000;
use constant ALL => 0b00001111;
use constant NEW => 0b00001000;
use constant NOT_NEW => 0b00000111;
use constant UPDATED => 0b00000100;
use constant NOT_UPDATED => 0b00001011;
use constant INSTALLED => 0b00000010;
use constant NOT_INSTALLED => 0b00001101;
use constant DOWNGRADED => 0b00000001;
use constant NOT_DOWNGRADED => 0b00001110;

my $ProcessDotNew;




####################################################################
############################ SESSION HANDLING ######################
####################################################################

my $user=$ENV{USER};
my $home=$ENV{HOME};
my $uid=getuid();
my $lockfile="/var/run/netpkg.pid";

$SIG{'INT'} = sub { unlink ("$lockfile") ; print "Killed by signal 2, bye...\n"; exit };
$SIG{'TERM'} = sub { unlink ("$lockfile") ; print "Killed by signal 15, bye...\n"; exit };
$SIG{'KILL'} = sub { unlink ("$lockfile") ; print "Killed by signal 9, bye...\n"; exit };
$SIG{'QUIT'} = sub { unlink ("$lockfile") ; print "Killed by signal 3, bye...\n"; exit };

if ( $uid ne 0 ) {
	print "Please launch me as root\n";
	exit;
}elsif( -e "$lockfile" ) {
	print "Another Netpkg seems to be running\n";
	exit;
}else{
	open(FH, ">$lockfile");
	print FH "$$";
	close(FH);
	chmod '0660', "$lockfile";
}


####################################################################
############################ CONFIG ###############################
####################################################################

# Internationalization
setlocale(LC_MESSAGES, "");
# my $GetText = Locale::gettext->domain("netpkg");
textdomain("netpkg");

# Retrieve configuration
my $configfile="/etc/netpkg.conf";

my %config = ( 	'Local_repository' => '/var/packages',
				'Package_logs' => '/var/log/packages',
				'Black_list' => 'aaa_base',
				'Keep_packages' => 'yes',
				'Protected_files' => '',
				'Netpkg_dir' => '/var/netpkg',
				'Handle_dependencies' => 'yes',
				'Proxy_Socket' => '',
				'Proxy_User' => '',
				'Proxy_Password' => '',
				'Logfile' => '/var/log/netpkg.log' );

my $Netpkg = Netpkg->new();

my %db = [];
my $Data = "";

my $option;
for $option ( keys %config ) {
	$config{$option} = $Netpkg->Configure($option, $config{$option}, $configfile); 
}

my %Mirrors; 
$Netpkg->GetMirrors(\%Mirrors, $configfile );

my @Blacklist = split ( / / , $Netpkg->{Black_list}); 

my $ZenStyle = TRUE ;

my $ShowOnly = $Netpkg->GetFilters;

if ( $ShowOnly == -1 ) { $ShowOnly = NONE } 

mkpath ("$Netpkg->{Netpkg_dir}", 0, 0755);
mkpath ("$Netpkg->{Netpkg_dir}/db", 0, 0755);


####################################################################
################################ GTK ###############################
####################################################################

my $WindowIconPath = "/usr/share/icons/hicolor/scalable/apps/netpkg.svg";

# Create windows
my $window = Gtk2::Window->new('toplevel') ;
my $window2 = Gtk2::Window->new('toplevel') ;

# Configure main window
$window->set_resizable(TRUE);
$window->set_default_size (700, 600);
$window->set_title("Netpkg - $version");
$window->signal_connect(destroy => \&MainExit);
$window->set_border_width(5);
$window->set_icon_from_file("$WindowIconPath");


# Tooltips ##############################

my $tooltips = Gtk2::Tooltips->new ;


# mirrors URL selector ##############################

my $UrlSelector = Gtk2::ComboBoxEntry->new_text;
$tooltips->set_tip ($UrlSelector, gettext("Enter the URL to mirror or choose one from the history. Mirrors which have been loaded properly will be auto-saved in the history"), FALSE);
for $_ ( sort { "$Mirrors{$a}" cmp "$Mirrors{$b}" } keys %Mirrors ) { $UrlSelector->append_text ($_) }
$UrlSelector->set_active (0);
my $Mirror = $UrlSelector->get_active_text;

my $icon1 = Gtk2::Image->new_from_icon_name ("gtk-refresh", 'GTK_ICON_SIZE_MENU');
my $RefreshButton = Gtk2::Button->new();
$RefreshButton->set_image ($icon1);
$tooltips->set_tip ($RefreshButton, gettext("Reload mirror content"), FALSE);
$RefreshButton->signal_connect(clicked => \&BuildDB);

my $MirrorBox = Gtk2::HBox->new(FALSE,5);
$MirrorBox->pack_start($UrlSelector, TRUE, TRUE, 5);
$MirrorBox->pack_start($RefreshButton, FALSE, TRUE, 5);

# info section ##############################
my $InfoLabel = Gtk2::Label->new;
$InfoLabel->set_width_chars(110);
$InfoLabel->set_justify('GTK_JUSTIFY_FILL');
$InfoLabel->set_line_wrap (TRUE);
$InfoLabel->set_line_wrap_mode ('word');
$InfoLabel->set_selectable(TRUE);
my $TextBuffer = gettext("Welcome to Netpkg !");
$InfoLabel->set_markup("<span foreground=\"DarkRed\" size=\"x-large\"><b>$TextBuffer</b></span>\n");

my $InfoBox = Gtk2::HBox->new(TRUE,0);
$InfoBox->set_border_width(7);
$InfoBox->pack_start($InfoLabel, TRUE, TRUE, 0);

my $InfoFrame = Gtk2::Frame->new('Information');
$InfoFrame->set_border_width(3);
$InfoFrame->add($InfoBox);

# Tree #########################################

#create a scrolled window that will host the treeview
my $ScrolledWindow = Gtk2::ScrolledWindow->new (undef, undef);
$ScrolledWindow->set_shadow_type ('etched-out');
$ScrolledWindow->set_policy ('automatic', 'automatic');
$ScrolledWindow->set_size_request (650, 260);
$ScrolledWindow->set_border_width(5);
my $TreeStore = Gtk2::TreeStore->new('Glib::String', 'Glib::String', 'Glib::String', 'Glib::String', 'Glib::Boolean');

	
#this will create a treeview, specify $TreeStore as its model
my $TreeView = Gtk2::TreeView->new($TreeStore);
my $select = $TreeView->get_selection();
$select->set_mode('single');
$select->signal_connect(changed => sub { RefreshInfoLabel($select, $TreeStore)});
$select->unselect_all;

#create a Gtk2::TreeViewColumn to add to $TreeView
my $column1 = Gtk2::TreeViewColumn->new();
$column1->set_title ("Category");
$column1->set_min_width(300);
my $icon = Gtk2::CellRendererPixbuf->new;	
$column1->pack_start ($icon, 0);	
$column1->set_attributes( $icon, 'icon-name' => 0 );	
my $text1 = Gtk2::CellRendererText->new;
$column1->pack_start ($text1, 0);
$column1->add_attribute ($text1, text => 1 );
$column1->set_clickable (TRUE);


my $column2 = Gtk2::TreeViewColumn->new();
$column2->set_title ("Available version");
$column2->set_min_width(150);
my $text2 = Gtk2::CellRendererText->new;
$column2->pack_start ($text2, 0);
$column2->add_attribute ($text2, text => 2 );  

my $column3 = Gtk2::TreeViewColumn->new();
$column3->set_title ("Installed version");
$column3->set_min_width(150);
my $text3 = Gtk2::CellRendererText->new;
$column3->pack_start ($text3, 0);
$column3->add_attribute ($text3, text => 3 );  	

my $column4 = Gtk2::TreeViewColumn->new();
$column4->set_title ("Select");
$column4->set_max_width(10);
my $toggle = Gtk2::CellRendererToggle->new;
$toggle->signal_connect (toggled => \&item_toggled, $TreeStore);
$column4->set_attributes($toggle);		
$column4->pack_start ($toggle, FALSE);
$column4->set_attributes($toggle, 'active' => 4 );
$column4->set_clickable (TRUE);


# add $columns to the treeview
$TreeView->append_column ($column1);
$TreeView->append_column ($column2);	
$TreeView->append_column ($column3);
$TreeView->append_column ($column4);		

# make a collumn searchable
$TreeView->set_search_column(1);

# Allow sorting on the column
$column1->set_sort_column_id(0);

$ScrolledWindow->add($TreeView);
	
# The frame around the tree  
my $TreeFrame = Gtk2::Frame->new('');
$TreeFrame->set_border_width(3);
$TreeFrame->add($ScrolledWindow);

# Filters #################################

my $ShowNew = Gtk2::CheckButton->new_with_label ( "New" );
$tooltips->set_tip ($ShowNew, "Show packages from mirror that are not installed yet", FALSE);
my $ShowInstalled = Gtk2::CheckButton->new_with_label ( "Installed" );
$tooltips->set_tip ($ShowInstalled, "Show already installed packages", FALSE);
my $ShowUpdated = Gtk2::CheckButton->new_with_label ( "Updates" );  
$tooltips->set_tip ($ShowUpdated, "Show packages from mirror with higher version than the installed version", FALSE);
my $ShowDowngraded = Gtk2::CheckButton->new_with_label ( "Downgrades" );
$tooltips->set_tip ($ShowDowngraded, "Show packages from mirror with lower version than the installed version", FALSE);

$ShowNew->set_active(TRUE) if ( $ShowOnly & NEW );
$ShowInstalled->set_active(TRUE) if ( $ShowOnly & INSTALLED );
$ShowUpdated->set_active(TRUE) if ( $ShowOnly & UPDATED );	
$ShowDowngraded->set_active(TRUE) if ( $ShowOnly & DOWNGRADED );

$ShowNew->signal_connect( toggled =>\&NewCallback) ; 
$ShowInstalled->signal_connect( toggled =>\&InstalledCallback) ;
$ShowUpdated->signal_connect( toggled =>\&UpdatedCallback);
$ShowDowngraded->signal_connect( toggled =>\&DowngradedCallback);


my $FilterBox = Gtk2::VBox->new;
$FilterBox->pack_start($ShowNew, TRUE, TRUE, 0);
$FilterBox->pack_start($ShowInstalled, TRUE, TRUE, 0);
$FilterBox->pack_start($ShowUpdated, TRUE, TRUE, 0);
$FilterBox->pack_start($ShowDowngraded, TRUE, TRUE, 0);

my $FilterFrame = Gtk2::Frame->new('Filters');
$FilterFrame->set_border_width(3);
$FilterFrame->add($FilterBox);
my $TopRightBox = Gtk2::VBox->new;
$TopRightBox->pack_start($FilterFrame, TRUE, TRUE, 0);

# Main action button ##################
my $icon2 = Gtk2::Image->new_from_icon_name ("netpkg", 'GTK_ICON_SIZE_DIALOG');
my $LaunchButton = Gtk2::Button->new();
$LaunchButton->set_image ($icon2);
$tooltips->set_tip ($LaunchButton, "Compute dependencies and launch the action window on selected packages to install / reinstall / upgrade / uninstall", FALSE);
$LaunchButton->signal_connect(clicked => \&ActionStart);

my $ActionBox = Gtk2::HBox->new(TRUE,5);
$ActionBox->pack_end($LaunchButton, TRUE, TRUE, 5);

# Progress bar ###########################

my $ProgressBar = Gtk2::ProgressBar->new;
$ProgressBar->set_fraction(0.0);
$ProgressBar->set_text("");
$ProgressBar->set_pulse_step(0.01);

my $icon3 = Gtk2::Image->new_from_icon_name ("exit", 'GTK_ICON_SIZE_MENU');
my $ExitButton = Gtk2::Button->new();
$ExitButton->set_image ($icon3);
$tooltips->set_tip ($ExitButton, "Click here to exit Netpkg! \n\nNetpkg is released under the terms of the Gnu Public Licence v2, Copyright Jean-Philippe Guillemin <jp.guillemin\@free.fr> \n bye :)", FALSE);
$ExitButton->signal_connect(clicked => \&MainExit) ; 

my $ProgressBox = Gtk2::HBox->new(FALSE,5);
$ProgressBox->pack_start($ProgressBar, TRUE, TRUE, 5);
$ProgressBox->pack_end($ExitButton, FALSE, TRUE, 5);

# Packing ################################
my $RightBox = Gtk2::VBox->new(FALSE,5);
$RightBox->pack_start($TopRightBox, TRUE, TRUE, 5);
$RightBox->pack_start($ActionBox, FALSE, FALSE, 5);

my $TreeBox = Gtk2::HBox->new;
$TreeBox->pack_start($TreeFrame, TRUE, TRUE, 0);
$TreeBox->pack_start($RightBox, FALSE, FALSE, 0);

my $OuterBox = Gtk2::VBox->new;
$OuterBox->pack_start($MirrorBox, FALSE, TRUE, 0);
$OuterBox->pack_start($TreeBox, FALSE, TRUE, 0);
$OuterBox->pack_start($InfoFrame, TRUE, TRUE, 0);
$OuterBox->pack_start($ProgressBox, FALSE, FALSE, 0);

$window->add($OuterBox);

# show window and enter GTK+2 loop
$window->show_all;

Gtk2->main_iteration while ( Gtk2->events_pending );

&BuildDB();

Gtk2->main;



####################################################################
############################ END MAIN ##############################
####################################################################


sub MainExit {
	unlink ("$lockfile") ;
	$Netpkg->SaveFilters($ShowOnly);
	$window->destroy;
	$window2->destroy;
	Gtk2->main_quit ;
	exit;
}


####################################################################
############################ PARSING ###############################
####################################################################

# Will download PACKAGES.TXT and parse it to generate %db
sub BuildDB {
	my $fh;
	my ($line,$package,$soft,$desc);
	
	$RefreshButton->set_sensitive( FALSE );
	
	$Data = "";
	%db = [];	

	$Mirror = $UrlSelector->get_active_text;
	my $RemoteFile = $Mirror."/PACKAGES.TXT" ;

	$ProgressBar->set_fraction(0);
	$ProgressBar->set_text("Connecting to mirror $Mirror");
	Gtk2->main_iteration while ( Gtk2->events_pending );
	
	my $result = $Netpkg->Get(
				$RemoteFile, 
				"mirror metadata",
				0,
				\$Data,
				"",
				$ProgressBar,
				0,
				0.8); 	
	
	if ( $result == 0 ) {
		for $_ ( keys %Mirrors ) { $Mirrors{$_} = 0 }
		$Mirrors{$Mirror} = 1;
		$Netpkg->SaveMirrors(\%Mirrors);
	}else{
		$ProgressBar->set_text("Failed connecting to $Mirror");		
		$ProgressBar->set_fraction(1);
		Gtk2->main_iteration while ( Gtk2->events_pending );	
		Gtk2->main_iteration while ( ! Gtk2->events_pending );	
		$ProgressBar->set_text("");
		$ProgressBar->set_fraction(0);
		Gtk2->main_iteration while ( Gtk2->events_pending );
		
		$RefreshButton->set_sensitive( TRUE );		
		return FALSE ;
	}
											
	$ProgressBar->set_text("Creating database");			
	Gtk2->main_iteration while ( Gtk2->events_pending );
	
	tie *SH, 'IO::Scalar', \$Data;
	while (<SH>) {
		
		Gtk2->main_iteration while ( Gtk2->events_pending );
		
		chomp $_;

		if ($_ =~ /PACKAGE NAME:\s*([^\s]*)$/) { 
			$package="$1"; 
			$package =~ /^(.*)-([^-]*)-[^-]*-([^-]*).tgz$/;
			$soft="$1"; 
			@{$db{$soft}}[AVAILABLE_PKG] = "$package";	
			@{$db{$soft}}[AVAILABLE_VERSION] = "$2";
			@{$db{$soft}}[AVAILABLE_BUILD] = "$3";		
			next;
		}
		if ($_ =~ /PACKAGE LOCATION:\s*\.?\/?([^\s]*)\s*$/) { 
			@{$db{$soft}}[PATH] = "$1";	
			
			if ( $ZenStyle = TRUE ) {
				if ( 
				@{$db{$soft}}[PATH] =~ /^l$/ || 
				@{$db{$soft}}[PATH] =~ /.*\/l$/ || 
				@{$db{$soft}}[PATH] =~ /^d$/ ||
				@{$db{$soft}}[PATH] =~ /.*\/d$/ 
				) {
					@{$db{$soft}}[CATEGORY] = "Libraries";
				}elsif ( 
				@{$db{$soft}}[PATH] =~ /^x$/ ||
				@{$db{$soft}}[PATH] =~ /.*\/x$/ 
				) {
					@{$db{$soft}}[CATEGORY] = "Xwindow";
				}elsif ( 
				@{$db{$soft}}[PATH] =~ /^x?ap$/ ||
				@{$db{$soft}}[PATH] =~ /.*\/x?ap$/ ||
				@{$db{$soft}}[PATH] =~ /.*\/kde$/ ||
				@{$db{$soft}}[PATH] =~ /.*\/g$/ 
				) {
					@{$db{$soft}}[CATEGORY] = "Applications";
				}elsif ( 
				@{$db{$soft}}[PATH] =~ /^a$/ || 
				@{$db{$soft}}[PATH] =~ /.*\/a$/ 
				) {
					@{$db{$soft}}[CATEGORY] = "System";
				}elsif ( 
				@{$db{$soft}}[PATH] =~ /^locale$/ || 
				@{$db{$soft}}[PATH] =~ /.*\/locale$/ 
				) {
					@{$db{$soft}}[CATEGORY] = "Localization";
				}elsif ( 
				@{$db{$soft}}[PATH] =~ /^games$/ || 
				@{$db{$soft}}[PATH] =~ /.*\/games$/ 
				) {
					@{$db{$soft}}[CATEGORY] = "Games";
				}elsif ( 
				@{$db{$soft}}[PATH] =~ /^n$/ || 
				@{$db{$soft}}[PATH] =~ /.*\/n$/ ||
				@{$db{$soft}}[PATH] =~ /^s$/ || 
				@{$db{$soft}}[PATH] =~ /.*\/s$/ 
				) {
					@{$db{$soft}}[CATEGORY] = "Network";
				}else{
					@{$db{$soft}}[CATEGORY] = "Misc";
				}
				
				#if ( @{$db{$soft}}[PATH] =~ /^extra.*/ ) {
				#	@{$db{$soft}}[CATEGORY] = "Extra ".@{$db{$soft}}[CATEGORY] ;
				#}
				
				
				@{$db{$soft}}[SORTSTRING] = @{$db{$soft}}[CATEGORY]."/".$package;
				
			}else{
				@{$db{$soft}}[CATEGORY] = @{$db{$soft}}[PATH] ;
				if ( @{$db{$soft}}[PATH] =~ /^extra.*/ ) {
					@{$db{$soft}}[SORTSTRING] = "y".@{$db{$soft}}[CATEGORY]."/".$package;
				}else{
					@{$db{$soft}}[SORTSTRING] = @{$db{$soft}}[CATEGORY]."/".$package;
				}
			}	
			
			next;
		}	
		if ($_ =~ /PACKAGE SIZE \(c.*:\s*(.*)\s*$/) { 
			@{$db{$soft}}[SIZE_COMPRESSED] = "$1";	
			next;						
		}			
		if ($_ =~ /PACKAGE SIZE \(u.*:\s*(.*)\s*$/) { 
			@{$db{$soft}}[SIZE_UNCOMPRESSED] = "$1";	
			next;							
		}														
		if ($_ =~ /PACKAGE REQUIRED:\s*([^\s]*)\s*$/) { 
			@{$db{$soft}}[DEPENDENCIES] = "$1";	
			next;				
		}
		# @{$db{$soft}}[DESCRIPTION] = "" if ( undef @{$db{$soft}}[DESCRIPTION] ) ;	
		while ($_ =~ /\Q${soft}\E:(.*)/) {
			$desc = $1;
			chomp $desc;
			$desc =~ s/[\ \t]{2,}/ /g ;						
			@{$db{$soft}}[DESCRIPTION] = @{$db{$soft}}[DESCRIPTION]." ".$desc;
			$_ = <SH>;
		}
		
		# Until we check local packages, everything is NEW
		@{$db{$soft}}[INSTALLED_VERSION] = "not installed";
		@{$db{$soft}}[STATUS] = STATUS_NEW ;
		
		# SELECTED PACKAGE BOOLEAN	
		@{$db{$soft}}[SELECTED_AS_PKG] = 0; 
		# SELECTED DEP BOOLEAN	
		@{$db{$soft}}[SELECTED_AS_DEP] = 0; 
		foreach (@Blacklist) {
		if ( $_ eq $soft ) {
			@{$db{$soft}}[BLACKLISTED] = 1; 
		}
	}
			
	}
	close(SH); 
	
	
	$ProgressBar->set_text("Creating local packages database");			
	$ProgressBar->set_fraction(0.9);
	Gtk2->main_iteration while ( Gtk2->events_pending );
	
	&BuildLocalDB();
	
	$ProgressBar->set_text("Building package tree");			
	$ProgressBar->set_fraction(0.95);	
	Gtk2->main_iteration while ( Gtk2->events_pending );
	$TreeStore->clear;
	&FillTree();
	
	$ProgressBar->set_text("Done");		
	$ProgressBar->set_fraction(1);
	Gtk2->main_iteration while ( Gtk2->events_pending );	
	Gtk2->main_iteration while ( ! Gtk2->events_pending );	
	$ProgressBar->set_text("");
	$ProgressBar->set_fraction(0);
	Gtk2->main_iteration while ( Gtk2->events_pending );
	
	$RefreshButton->set_sensitive( TRUE );

}

####################################################################
# Will check installed packages to fill %db related fields
sub BuildLocalDB {
	my $fh;
	my ($package,$soft,$version,$build,$desc,$meta);

	my %localdb = [];
	opendir(DH, "$Netpkg->{Package_logs}") || die "Cannot opendir $Netpkg->{Package_logs} : $!";
	foreach (sort readdir(DH)) {
		$meta = $_;
		next if ( length ($meta) <= 3 );
		$package = "$meta".".tgz";
		$package =~ /^(.*)-([^-]*)-[^-]*-([^-]*).tgz$/;
		$soft="$1"; 
		$version="$2";
		$build="$3";
		
		@{ $db{$soft} }[INSTALLED_PKG] = "$package";	
		@{ $db{$soft} }[INSTALLED_VERSION] = "$version";	
		@{ $db{$soft} }[INSTALLED_BUILD] = "$build";			
		
		if (defined @{$db{$soft}}[AVAILABLE_PKG] ) {
			# STATUS ( -1:downgraded ; 0:installed ; 1:updated )
			@{$db{$soft}}[STATUS] = &ZW::Netpkg::PVfilter::test_package("@{$db{$soft}}[AVAILABLE_PKG]", "@{$db{$soft}}[INSTALLED_PKG]") ;					
		}else{
			@{$db{$soft}}[STATUS] = STATUS_OK;
			@{$db{$soft}}[AVAILABLE_VERSION] = "not on mirror";	
			@{$db{$soft}}[CATEGORY] = "Local";	
			@{$db{$soft}}[SORTSTRING] = "z"."/".@{$db{$soft}}[CATEGORY]."/".$package;
			open(FH, "<$Netpkg->{Package_logs}/$meta");
			while(<FH>) {
				if ($_ =~ /^COMPRESSED PACKAGE SIZE:\s*(.*)\s*$/) { 
					@{$db{$soft}}[SIZE_COMPRESSED] = "$1";	
					next;				
				}			
				if ($_ =~ /^UNCOMPRESSED PACKAGE SIZE:\s*(.*)\s*$/) { 
					@{$db{$soft}}[SIZE_UNCOMPRESSED] = "$1";
					next;							
				}	
				if ($_ =~ /^PACKAGE LOCATION:\s*(.*)\s*$/) { 
					my $fullpath = "$1";
					my $regex = '^'.$Netpkg->{Local_repository}.'/(.*)/[^/]*.tgz\s*$';
					$regex =~ s/\//\\\//g ;
					if ($fullpath =~ /$regex/ ) {
						@{$db{$soft}}[PATH] = "$1";
					}else{
						@{$db{$soft}}[PATH] = "local";
					}
					next;							
				}					
				
				# @{$db{$soft}}[DESCRIPTION] = "" if ( undef @{$db{$soft}}[DESCRIPTION] ) ;	
				while ($_ =~ /\Q${soft}\E:(.*)/) {
					$desc = $1;
					chomp $desc;
					$desc =~ s/[\ \t]{2,}/ /g ;									
					@{$db{$soft}}[DESCRIPTION] = @{$db{$soft}}[DESCRIPTION]." ".$desc;
					$_ = <FH>;
				}
			}
			close(FH);
		}		
	}
	closedir(DH); 
}

####################################################################
# Will build / rebuild the treestore from %db data
sub FillTree {
	
	#fill the tree with %db data
	my ($previous,$soft);
	my $iter;
	my $iter_child;
	for $soft ( sort { "@{$db{$a}}[SORTSTRING]" cmp "@{$db{$b}}[SORTSTRING]" } keys %db ) {

		if (defined @{$db{$soft}}[CATEGORY]) {
			
			# print "@{$db{$soft}}[AVAILABLE_PKG]" . " " . "@{$db{$soft}}[AVAILABLE_VERSION]" . "->" . "@{$db{$soft}}[INSTALLED_VERSION]" . "\n";
			
	
			if ($previous ne "@{$db{$soft}}[CATEGORY]") {
				$iter = $TreeStore->append(undef);
				$TreeStore->set ($iter, 0, "gtk-directory", 1, "@{$db{$soft}}[CATEGORY]", 2, "", 3, "", 4, @{$db{$soft}}[SELECTED_AS_PKG]);
			}
			
			if ( @{$db{$soft}}[STATUS] == STATUS_OK && $ShowOnly & INSTALLED ) {
				$iter_child = $TreeStore->append($iter);
				$TreeStore->set ($iter_child, 0, "netpkg-ins", 1, "$soft", 2, "@{$db{$soft}}[AVAILABLE_VERSION]", 3, "@{$db{$soft}}[INSTALLED_VERSION]", 4, @{$db{$soft}}[SELECTED_AS_PKG]);
			}elsif ( @{$db{$soft}}[STATUS] == STATUS_DOWNGRADED && $ShowOnly & DOWNGRADED ) {
				$iter_child = $TreeStore->append($iter);
				$TreeStore->set ($iter_child, 0, "netpkg-down", 1, "$soft", 2, "@{$db{$soft}}[AVAILABLE_VERSION]", 3, "@{$db{$soft}}[INSTALLED_VERSION]", 4, @{$db{$soft}}[SELECTED_AS_PKG]);
			}elsif ( @{$db{$soft}}[STATUS] == STATUS_UPDATED && $ShowOnly & UPDATED ) {
				$iter_child = $TreeStore->append($iter);
				$TreeStore->set ($iter_child, 0, "netpkg-upd", 1, "$soft", 2, "@{$db{$soft}}[AVAILABLE_VERSION]", 3, "@{$db{$soft}}[INSTALLED_VERSION]", 4, @{$db{$soft}}[SELECTED_AS_PKG]);					
			}elsif ( @{$db{$soft}}[STATUS] == STATUS_NEW && $ShowOnly & NEW ) {
				$iter_child = $TreeStore->append($iter);
				$TreeStore->set ($iter_child, 0, "package", 1, "$soft", 2, "@{$db{$soft}}[AVAILABLE_VERSION]", 3, "@{$db{$soft}}[INSTALLED_VERSION]", 4, @{$db{$soft}}[SELECTED_AS_PKG]);
			}
			$previous = @{$db{$soft}}[CATEGORY];	

		}
	}
}



####################################################################
############################ CALLBACKS #############################
####################################################################

# called when a row check button is selected
sub item_toggled {
	my ($cell, $path_str, $store, $child_iter, $n_child ) = @_;
  
	my $path = Gtk2::TreePath->new_from_string ($path_str);

	# toggle iter
	my $iter = $store->get_iter ($path);
	if (defined $iter) {
		my $boolean = $store->get ($iter, 4);
		my $soft = $store->get ($iter, 1);
		#print "$soft\n";
		$boolean =!$boolean;
		$store->set ($iter, 4, $boolean);	
		@{$db{$soft}}[SELECTED_AS_PKG] = $boolean;	
			
		if ( $store->iter_has_child ($iter) ) {
			my $childrens = $store->iter_n_children ($iter);
			# print "$childrens\n";
			for ($n_child = 0; $n_child <= $childrens - 1; $n_child++) {
				$child_iter = $store->iter_nth_child ($iter, $n_child);
				$store->set ($child_iter, 4, $boolean);	
				my $soft = $store->get ($child_iter, 1);
				@{$db{$soft}}[SELECTED_AS_PKG] = $boolean;
			}
		}
	}
}


####################################################################
# called when the corresponding filter is checked 
sub NewCallback {
	my $togglebutton = $_[0];  
	if ( $togglebutton->get_active ) {  
		$ShowOnly = $ShowOnly | NEW ;
	} else {  
		$ShowOnly = $ShowOnly & NOT_NEW ;
	}  

	$TreeStore->clear;
	&FillTree();
	$Netpkg->SaveFilters($ShowOnly);
	return TRUE ;
}

####################################################################
# called when the corresponding filter is checked 
sub InstalledCallback {
	my $togglebutton = $_[0];  
	if ( $togglebutton->get_active ) {  
		$ShowOnly = $ShowOnly | INSTALLED ;
	} else {  
		$ShowOnly = $ShowOnly & NOT_INSTALLED ;
	}  

	$TreeStore->clear;
	&FillTree();
	$Netpkg->SaveFilters($ShowOnly);
	return TRUE ;
}

####################################################################
# called when the corresponding filter is checked 
sub UpdatedCallback {
	my $togglebutton = $_[0];  
	if ( $togglebutton->get_active ) {  
		$ShowOnly = $ShowOnly | UPDATED ;
	} else {  
		$ShowOnly = $ShowOnly & NOT_UPDATED ;
	}  
	$TreeStore->clear;
	&FillTree();
	$Netpkg->SaveFilters($ShowOnly);
	return TRUE ;
}

####################################################################
# called when the corresponding filter is checked 
sub DowngradedCallback {
	my $togglebutton = $_[0];  
	if ( $togglebutton->get_active ) {  
		$ShowOnly = $ShowOnly | DOWNGRADED ;
	} else {  
		$ShowOnly = $ShowOnly & NOT_DOWNGRADED ;
	}  
	
	$TreeStore->clear;
	&FillTree();
	$Netpkg->SaveFilters($ShowOnly);
	return TRUE ;
}

####################################################################
# called when a row is selected
sub RefreshInfoLabel {
	my ($selection, $store) = @_; 
	my $iter = $selection->get_selected;
	if ( defined $iter ) {
		my $soft = $store->get ($iter, 1);
		if ( ($store->get ($iter, 2)) ne "" || ($store->get ($iter, 3)) ne "" ) {
			# Taking care of deps if any
			my $depline;
			if ( @{$db{$soft}}[CATEGORY] ne "Local" ) {
				my @Deps = split ("," , @{$db{$soft}}[DEPENDENCIES]);
				
				my $InstalledDeps;
				my $MissingDeps;
				my $Dep;
				
				foreach $Dep(@Deps) {
					if ( @{$db{$Dep}}[STATUS] == STATUS_OK ) {
						$InstalledDeps = $InstalledDeps.$Dep." ";
					}else{
						$MissingDeps = $MissingDeps.$Dep." ";
					}
				}

				if ($InstalledDeps =~ /^[ \t]*$/ ) { $InstalledDeps = "none" }
				if ($MissingDeps =~ /^[ \t]*$/ ) { $MissingDeps = "none" }
				
			$depline = "<span size=\"medium\"><b>Installed dependencies: </b></span><span foreground=\"Blue\" size=\"small\"> ".$InstalledDeps."</span>\n".
			"<span size=\"medium\"><b>Missing or updated dependencies: </b></span><span foreground=\"Red\" size=\"small\"> ".$MissingDeps."</span>\n" ;
			}
			
			
			my $mainline;
			if ( @{$db{$soft}}[BLACKLISTED] == 1 ) {
				print "$soft\n" if ( $debug == 1 ) ;
				$mainline = "<span foreground=\"DarkGrey\" size=\"x-large\"><b>".$soft." (blacklisted)</b></span>\n";
			} else {
				if ( ! defined @{$db{$soft}}[AVAILABLE_PKG] ) {
					$mainline = "<span foreground=\"Orange\" size=\"x-large\"><b>".$soft."</b></span>\n";
				}else{
					$mainline = "<span foreground=\"DarkRed\" size=\"x-large\"><b>".$soft."</b></span>\n";	
				}
			}
			
			my $versionline;
			if ( defined @{$db{$soft}}[AVAILABLE_PKG] ) {
				$versionline = "<span size=\"medium\"><b>Available version: </b>".@{$db{$soft}}[AVAILABLE_VERSION]." - build ".@{$db{$soft}}[AVAILABLE_BUILD]."</span>\n";		
			}
			if ( defined @{$db{$soft}}[INSTALLED_PKG] ) {
				$versionline = $versionline."<span size=\"medium\"><b>Installed version: </b>".@{$db{$soft}}[INSTALLED_VERSION]." - build ".@{$db{$soft}}[INSTALLED_BUILD]."</span>\n";		
			}
			
			# We encode unsafe characters in HTML entities
			my $HTMLdesc = encode_entities_numeric(@{$db{$soft}}[DESCRIPTION]);
			
			my $Markup = 
			$mainline.$versionline.
			"<span size=\"medium\"><b>Description: </b></span><span size=\"small\"> ".$HTMLdesc."</span>\n".
			$depline.
			"<span size=\"medium\"><b>Location: </b>".@{$db{$soft}}[PATH]."</span>\n".
			"<span size=\"medium\"><b>Package size: </b>".@{$db{$soft}}[SIZE_COMPRESSED]."</span>\n".
			"<span size=\"medium\"><b>Install size: </b>".@{$db{$soft}}[SIZE_UNCOMPRESSED]."</span>\n";		
			$InfoLabel->set_markup($Markup);
		}else{
			$InfoLabel->set_markup("<span foreground=\"DarkGrey\" size=\"x-large\"><b>".$soft."</b></span>\n");
		}
	}else{
		$InfoLabel->set_text("");
	}
	return TRUE ;
}

####################################################################
# Called when the big Action button is pressed
sub ActionStart {
	
	$Netpkg->ProcessDeps(\%db);
	

	# Reporting current selections and related deps
	my $soft;
	my $PkgList;
	my $DepList;	
	my $LocalList;
	my $Npkgs = 0;
	my $Ndeps = 0;
	my $Nlocal = 0;	
	for $soft ( keys %db ) {
		
		next if ( @{$db{$soft}}[BLACKLISTED] );
		
		if ( defined @{$db{$soft}}[INSTALLED_PKG] ) { 
			if ( @{$db{$soft}}[SELECTED_AS_PKG] ) {
				$Nlocal++;				
				if ( $Nlocal % 3 == 0 ){
					$LocalList = $LocalList.@{$db{$soft}}[INSTALLED_PKG]."\n";
				}else{
					$LocalList = $LocalList.@{$db{$soft}}[INSTALLED_PKG]." ";
				}	
			}	
		}		
		if ( defined @{$db{$soft}}[AVAILABLE_PKG] ) { 
			if ( @{$db{$soft}}[SELECTED_AS_PKG] ) {
				$Npkgs++;
				@{$db{$soft}}[SELECTED_AS_DEP] = 0;		
				if ( $Npkgs % 3 == 0 ){
					$PkgList = $PkgList.@{$db{$soft}}[AVAILABLE_PKG]."\n";
				}else{
					$PkgList = $PkgList.@{$db{$soft}}[AVAILABLE_PKG]." ";
				}
				next;
			}
			if ( @{$db{$soft}}[SELECTED_AS_DEP] && @{$db{$soft}}[STATUS] != STATUS_OK ) {
				$Ndeps++;	
				if ( $Ndeps % 3 == 0 ){
					$DepList = $DepList.@{$db{$soft}}[AVAILABLE_PKG]."\n";
				}else{
					$DepList = $DepList.@{$db{$soft}}[AVAILABLE_PKG]." ";
				}		
			}
		}

	}
	
	my $PkgListMarkup;
	if ( $Npkgs == 0 ) { 
		$PkgListMarkup = "";
	}else{
		$PkgListMarkup = "<span size=\"medium\"><b>Available packages: </b></span>\n"."<span size=\"small\"> ".$PkgList."</span>\n" ;
	}
	
	my $DepListMarkup;
	if ( $Ndeps == 0 ) { 
		$DepListMarkup = "";
	}else{
		$DepListMarkup = "<span size=\"medium\"><b>Missing or updated dependencies: </b></span>\n"."<span size=\"small\"> ".$DepList."</span>\n" ;	
	}	

	my $LocalListMarkup;
	if ( $Nlocal == 0 ) { 
		$LocalListMarkup = "";
	}else{
		$LocalListMarkup = "<span size=\"medium\"><b>Installed packages: </b></span>\n"."<span size=\"small\"> ".$LocalList."</span>\n" ;	
	}	
	
	if ( $Npkgs > 0 || $Nlocal > 0 ) { 
		
		$LaunchButton->set_sensitive( FALSE );
		$ScrolledWindow->set_sensitive( FALSE );
		$RefreshButton->set_sensitive( FALSE );
		
		# Configuring the window
		$window2->signal_connect(destroy => \&ActionExit);
		$window2->set_default_size (500, 150);
		$window2->set_resizable(TRUE);
		$window2->set_title("Actions");
		$window2->set_border_width(7);
		$window2->set_icon_from_file("$WindowIconPath");
		
		
		# info section
		my $InfoLabel = Gtk2::Label->new;
		$InfoLabel->set_width_chars(100);
		$InfoLabel->set_justify('GTK_JUSTIFY_FILL');
		$InfoLabel->set_line_wrap (TRUE);
		$InfoLabel->set_line_wrap_mode ('word');

		my $InfoBox = Gtk2::HBox->new(TRUE,0);
		$InfoBox->set_border_width(7);
		$InfoBox->pack_start($InfoLabel, TRUE, TRUE, 0);

		my $InfoFrame = Gtk2::Frame->new('Selection');
		$InfoFrame->set_border_width(3);
		$InfoFrame->add($InfoBox);	
		
		my $Markup = $PkgListMarkup . $DepListMarkup . $LocalListMarkup ;	

		$InfoLabel->set_markup($Markup);

		# Actions ##################################"
		my $ExitButton = Gtk2::Button->new_from_stock('gtk-close');
		$ExitButton->signal_connect(clicked => sub{ $window2->destroy; } ) ; 
		my $icon1 = Gtk2::Image->new_from_icon_name ("gtk-save", 'GTK_ICON_SIZE_DIALOG');
		my $InstallButton = Gtk2::Button->new_with_mnemonic( "Install available" );
		$InstallButton->set_image ($icon1);
		$InstallButton->signal_connect(clicked => \&ProcessPkgs);
		
		my $icon2 = Gtk2::Image->new_from_icon_name ("gtk-save-all", 'GTK_ICON_SIZE_DIALOG');
		my $InstallDepsButton = Gtk2::Button->new_with_mnemonic( "Include deps" );
		$InstallDepsButton->set_image ($icon2);
		$InstallDepsButton->signal_connect(clicked => \&ProcessPkgsAndDeps);

		my $icon3 = Gtk2::Image->new_from_icon_name ("user-trash", 'GTK_ICON_SIZE_DIALOG');
		my $RemoveButton = Gtk2::Button->new_with_mnemonic( "Remove installed" );
		$RemoveButton->set_image ($icon3);
		$RemoveButton->signal_connect(clicked => \&UninstallPkgs);
		
		
		my $CheckDotNew = Gtk2::CheckButton->new_with_label ( "Accept .new config files" );  
		$CheckDotNew->signal_connect( toggled =>\&CheckDotNewCallback);
		
		
		my $ActionBox = Gtk2::HBox->new;		
		if ( $Npkgs > 0 ) { $ActionBox->pack_start($InstallButton, FALSE, FALSE, 0); }
		if ( $Ndeps > 0 ) { $ActionBox->pack_start($InstallDepsButton, FALSE, FALSE, 0); }
		if ( $Nlocal > 0 ) { $ActionBox->pack_start($RemoveButton, FALSE, FALSE, 0); }	
		$ActionBox->pack_end($ExitButton, FALSE, FALSE, 0);
		if ( $Nlocal < $Npkgs ) { $ActionBox->pack_end($CheckDotNew, FALSE, FALSE, 0); }

		my $OuterBox = Gtk2::VBox->new;
		$OuterBox->pack_start($InfoFrame, TRUE, TRUE, 0);
		$OuterBox->pack_start($ActionBox, FALSE, FALSE, 0);
		$window2->add($OuterBox);		
		$window2->show_all;
	}	
	
	return TRUE ;
}

####################################################################
# Called when the .new checkbutton is checked/unchecked
sub  CheckDotNewCallback {
	my ( $checkbutton) = @_ ;
	$ProcessDotNew = $checkbutton->get_active ;
	# print "$ProcessDotNew\n";
}

####################################################################
# Called at Action window exit to enable Action button
sub ActionExit {
	# $window2->destroy ;
	$LaunchButton->set_sensitive( TRUE );
	$ScrolledWindow->set_sensitive( TRUE );
	$RefreshButton->set_sensitive( TRUE );
	return TRUE ;
}

####################################################################
# Will download and install packages WITHOUT deps
sub ProcessPkgs{
	$window2->destroy;
	$TreeView->collapse_all;
	$ScrolledWindow->set_sensitive( FALSE );
	$LaunchButton->set_sensitive( FALSE );
	$RefreshButton->set_sensitive( FALSE );
	$InfoLabel->set_markup("<span foreground=\"DarkRed\" size=\"x-large\"><b>Processing packages ...</b></span>\n");
	&DownloadPkgs (SELECTED_AS_PKG);
	&InstallPkgs (SELECTED_AS_PKG);
	&CheckInstallResult();
	$LaunchButton->set_sensitive( TRUE );
	$ScrolledWindow->set_sensitive( TRUE );
	$RefreshButton->set_sensitive( TRUE );
	$InfoLabel->set_text("");
	return TRUE ;		
}

####################################################################
# Will download and install packages AND deps
sub ProcessPkgsAndDeps{
	$window2->destroy ;
	$TreeView->collapse_all;
	$LaunchButton->set_sensitive( FALSE );
	$ScrolledWindow->set_sensitive( FALSE );
	$RefreshButton->set_sensitive( FALSE );
	$InfoLabel->set_markup("<span foreground=\"DarkRed\" size=\"x-large\"><b>Processing packages & deps ...</b></span>\n");
	&DownloadPkgs (SELECTED_AS_PKG);
	&DownloadPkgs (SELECTED_AS_DEP);	
	&InstallPkgs (SELECTED_AS_PKG);
	&InstallPkgs (SELECTED_AS_DEP);		
	&CheckInstallResult();
	$LaunchButton->set_sensitive( TRUE );
	$ScrolledWindow->set_sensitive( TRUE );
	$RefreshButton->set_sensitive( TRUE );
	$InfoLabel->set_text("");
	return TRUE ;		
}


####################################################################
# Will download packages OR deps depending on the value of $SelectedAsType
# DownloadPkgs($SelectedAsType);
sub DownloadPkgs {

	my $SelectedAsType = shift ;
	print "DownloadPkgs SelectedAsType = $SelectedAsType\n" if ( $debug == 1 ) ;
	$InfoLabel->set_text("");
	
	my $Npkgs = 0 ;
	for my $soft ( keys %db ) {
		if ( defined @{$db{$soft}}[AVAILABLE_PKG] ) { 
			if ( @{$db{$soft}}[$SelectedAsType] == 1 ) {	
				print "DownloadPkgs soft = $soft\n" if ( $debug == 1 ) ;
				$Npkgs++;				
			}
		}
	}
	print "DownloadPkgs Npkgs = $Npkgs\n" if ( $debug == 1 ) ;
	if ($Npkgs == 0) { $Npkgs = 1 }

	$ProgressBar->set_fraction(0);
	
	my $Range = 0.99 / $Npkgs  ;
	my $Progress = 0;
	$ProgressBar->set_fraction($Progress);
	Gtk2->main_iteration while ( Gtk2->events_pending );
	my $soft;
	for $soft ( keys %db ) {
		if ( defined @{$db{$soft}}[AVAILABLE_PKG] ) { 
			if ( @{$db{$soft}}[$SelectedAsType] == 1 ) {
				
				
				my $Path = $Netpkg->{Local_repository}."/".@{$db{$soft}}[PATH] ;
				my $File = $Netpkg->{Local_repository}."/".@{$db{$soft}}[PATH]."/".@{$db{$soft}}[AVAILABLE_PKG] ;
				
				if ( -e $File ) {
					$ProgressBar->set_text("Skipping @{$db{$soft}}[AVAILABLE_PKG]");
					print "Skipping @{$db{$soft}}[AVAILABLE_PKG]" if ( $debug == 1 ) ;
					Gtk2->main_iteration while ( Gtk2->events_pending );
				}else{
					my $RemoteFile = $Mirror."/".@{$db{$soft}}[PATH]."/".@{$db{$soft}}[AVAILABLE_PKG] ;
					mkpath ("$Path", 0, 0755);
					$ProgressBar->set_text("Loading @{$db{$soft}}[AVAILABLE_PKG]");
					print "Loading @{$db{$soft}}[AVAILABLE_PKG]" if ( $debug == 1 ) ;
					Gtk2->main_iteration while ( Gtk2->events_pending );
					my $buffer = "" ;
					
					my $result = $Netpkg->Get(
								$RemoteFile, 
								@{$db{$soft}}[AVAILABLE_PKG], 
								1,
								\$buffer,
								$File,
								$ProgressBar,
								$Progress,
								$Range); 	
								
					if ( ! $result == 0 ) {
						$ProgressBar->set_text("Failed to download @{$db{$soft}}[AVAILABLE_PKG]");		
						Gtk2->main_iteration while ( Gtk2->events_pending );	
						Gtk2->main_iteration while ( ! Gtk2->events_pending );	
						$Progress = $Progress + $Range ;
						delete($db{$soft});
						Gtk2->main_iteration while ( Gtk2->events_pending );
						next;
					}	
								
				}
			$Progress = $Progress + $Range ;
			}
		}
	}	
	$ProgressBar->set_fraction(1);
	$ProgressBar->set_text("Done");

	return TRUE ;	
}

####################################################################
# Will install/update/reinstall packages OR deps depending on the value of $SelectedAsType
# InstallPkgs($SelectedAsType);
sub InstallPkgs {
	
	
	my $SelectedAsType = shift ;
	print "InstallPkgs SelectedAsType = $SelectedAsType\n" if ( $debug == 1 ) ;
	
	my $Npkgs = 0 ;
	my $soft;
	for $soft ( keys %db ) {
		if ( defined @{$db{$soft}}[AVAILABLE_PKG] ) { 
			if ( @{$db{$soft}}[$SelectedAsType] == 1 ) {
				print "InstallPkgs soft = $soft\n" if ( $debug == 1 ) ;
				$Npkgs++;				
			}
		}
	}
	print "InstallPkgs Npkgs = $Npkgs\n" if ( $debug == 1 ) ;
	if ($Npkgs == 0) { $Npkgs = 1 }
	
	$ProgressBar->set_fraction(0);
	my $Range = 0.87 / $Npkgs  ;
	my $Progress = 0.03;
	$ProgressBar->set_fraction($Progress);
	Gtk2->main_iteration while ( Gtk2->events_pending );
	
	for $soft ( keys %db ) {
		if ( defined @{$db{$soft}}[AVAILABLE_PKG] ) { 
			if ( @{$db{$soft}}[$SelectedAsType] == 1 ) {
			
				my $Path = $Netpkg->{Local_repository}."/".@{$db{$soft}}[PATH] ;
				my $File = $Netpkg->{Local_repository}."/".@{$db{$soft}}[PATH]."/".@{$db{$soft}}[AVAILABLE_PKG] ;
				
				if ( @{$db{$soft}}[STATUS] == STATUS_OK ) {
					$ProgressBar->set_text("Reinstalling @{$db{$soft}}[AVAILABLE_PKG]");
					$Netpkg->Reinstall( $File);				
				}elsif ( @{$db{$soft}}[STATUS] == STATUS_DOWNGRADED ) {
					$ProgressBar->set_text("Downgrading @{$db{$soft}}[AVAILABLE_PKG]");
					$Netpkg->Update( $File);
				}elsif ( @{$db{$soft}}[STATUS] == STATUS_UPDATED ) {
					$ProgressBar->set_text("Updating @{$db{$soft}}[AVAILABLE_PKG]");
					$Netpkg->Update( $File);
				}elsif ( @{$db{$soft}}[STATUS] == STATUS_NEW ) {
					$ProgressBar->set_text("Installing @{$db{$soft}}[AVAILABLE_PKG]");
					$Netpkg->Install( $File);
				}
				
			$Progress = $Progress + $Range ;
			$ProgressBar->set_fraction($Progress);
			Gtk2->main_iteration while ( Gtk2->events_pending );
			}
		}
	}	
	
	$ProgressBar->set_text("Refreshing database");			
	$ProgressBar->set_fraction(0.9);
	Gtk2->main_iteration while ( Gtk2->events_pending );
	
	&BuildLocalDB();
				
	$ProgressBar->set_fraction(0.95);	
	Gtk2->main_iteration while ( Gtk2->events_pending );
	$TreeStore->clear;
	&FillTree();
	
	$ProgressBar->set_fraction(1);
	$ProgressBar->set_text("Done");
	Gtk2->main_iteration while ( ! Gtk2->events_pending );	
	$ProgressBar->set_fraction(0);
	$ProgressBar->set_text("");
	
	# Check for any .new files
	if ( $ProcessDotNew ) { &DotNew }
	
	return TRUE ;	
}

####################################################################
# Will check the result of the install process

sub CheckInstallResult {
	
	my $soft;
	my 	$BadPkgs = 0;
	my 	$GoodPkgs = 0;
	my 	$GoodList;
	my 	$BadList;
		
	for $soft ( keys %db ) {
		if ( defined @{$db{$soft}}[INSTALLED_PKG] ) { 
			next if ( @{$db{$soft}}[BLACKLISTED] );
			
			if ( @{$db{$soft}}[SELECTED_AS_PKG] == 1 || @{$db{$soft}}[SELECTED_AS_DEP] == 1 ) { 
				if ( @{$db{$soft}}[AVAILABLE_VERSION] eq @{$db{$soft}}[INSTALLED_VERSION] ) {
					$GoodPkgs++;
					if ( $GoodPkgs % 3 == 0 ){
						$GoodList = $GoodList.@{$db{$soft}}[INSTALLED_PKG]."\n";
					}else{
						$GoodList = $GoodList.@{$db{$soft}}[INSTALLED_PKG]." ";
					}
				}else{
					$BadPkgs++;	
					if ( $BadPkgs % 3 == 0 ){
						$BadList = $BadList.@{$db{$soft}}[INSTALLED_PKG]." ";
					}else{
						$BadList = $BadList.@{$db{$soft}}[INSTALLED_PKG]." ";
					}				
				}	
			}
		}
		
		# Switching off the selection
		@{$db{$soft}}[SELECTED_AS_PKG] = 0 ;
		@{$db{$soft}}[SELECTED_AS_DEP] = 0 ;
	}
	
	my $GoodListMarkup;
	if ( $GoodPkgs == 0 ) { 
		$GoodListMarkup = "";
	}else{
		$GoodListMarkup = "<span size=\"medium\"><b>Successfully installed: </b></span>\n"."<span size=\"small\"> ".$GoodList."</span>\n" ;
	}
	
	my $BadListMarkup;
	if ( $BadPkgs == 0 ) { 
		$BadListMarkup = "";
	}else{
		$BadListMarkup = "<span size=\"medium\"><b>Failed to install: </b></span>\n"."<span size=\"small\"> ".$BadList."</span>\n" ;	
	}
	
	
	if ( $GoodPkgs > 0 || $BadPkgs > 0 ) { 
		
	
		# Configuring the window
		#$window2->signal_connect(destroy => \&ActionExit);
		$window2->set_default_size (500, 150);
		$window2->set_resizable(TRUE);
		$window2->set_title("Report");
		$window2->set_border_width(7);
		$window2->set_icon_from_file("$WindowIconPath");
		
		
		# info section
		my $InfoLabel = Gtk2::Label->new;
		$InfoLabel->set_width_chars(100);
		$InfoLabel->set_justify('GTK_JUSTIFY_FILL');
		$InfoLabel->set_line_wrap (TRUE);
		$InfoLabel->set_line_wrap_mode ('word');

		my $InfoBox = Gtk2::HBox->new(TRUE,0);
		$InfoBox->set_border_width(7);
		$InfoBox->pack_start($InfoLabel, TRUE, TRUE, 0);

		my $InfoFrame = Gtk2::Frame->new('Install process result');
		$InfoFrame->set_border_width(3);
		$InfoFrame->add($InfoBox);	
		
		my $Markup = $GoodListMarkup . $BadListMarkup ;	

		$InfoLabel->set_markup($Markup);

		# Actions ##################################"
		my $ExitButton = Gtk2::Button->new_from_stock('gtk-close');
		$ExitButton->signal_connect(clicked => sub{ $window2->destroy; } ) ; 
				
		my $ActionBox = Gtk2::HBox->new;		
		$ActionBox->pack_end($ExitButton, FALSE, FALSE, 0);

		my $OuterBox = Gtk2::VBox->new;
		$OuterBox->pack_start($InfoFrame, TRUE, TRUE, 0);
		$OuterBox->pack_start($ActionBox, FALSE, FALSE, 0);
		$window2->add($OuterBox);		
		$window2->show_all;
	}	
	
	return TRUE ;	
}

####################################################################
# Will search .new files in /etc and rename them
sub DotNew{
	
	# Check for any .new files
	
	use vars qw/*name *dir *prune/;
	*name   = *File::Find::name;
	*dir    = *File::Find::dir;
	*prune  = *File::Find::prune;
	
	my %DotNews;
	
	File::Find::find({wanted => sub { $DotNews{"$name"} = $dir if (/^.*\.new\z/s)   } }, '/etc');	
	
	my $DotNewFile;
	for $DotNewFile ( keys %DotNews ) { 
		$DotNewFile =~ /^(.*)\.new$/ ;
		my $NewFile = $1;
		# print $DotNewFile." -> ".$NewFile."\n";
		rename ("$DotNewFile", "$NewFile");
	}
	
	return TRUE ;	
}

####################################################################
# Will uninstall selected packages if they are installed ;)
sub UninstallPkgs{
	$window2->destroy ;
	$TreeView->collapse_all;
	$LaunchButton->set_sensitive( FALSE );
	$ScrolledWindow->set_sensitive( FALSE );
	$RefreshButton->set_sensitive( FALSE );
	$InfoLabel->set_markup("<span foreground=\"DarkRed\" size=\"x-large\"><b>Removing packages ...</b></span>\n");
	
	my $Npkgs = 0 ;
	my 	$PkgList;
	my $soft;
	for $soft ( keys %db ) {
		if ( defined @{$db{$soft}}[INSTALLED_PKG] ) { 
			if ( @{$db{$soft}}[SELECTED_AS_PKG] ) {
				$Npkgs++;	
				if ( $Npkgs % 3 == 0 ){
					$PkgList = $PkgList.@{$db{$soft}}[INSTALLED_PKG]."\n";
				}else{
					$PkgList = $PkgList.@{$db{$soft}}[INSTALLED_PKG]." ";
				}			
			}
		}
	}
	
	my $Markup;
	if ( $Npkgs == 0 ) { 
		$Markup = "";
	}else{
		$Markup = "<span size=\"medium\"><b>Uninstalled packages: </b></span>\n"."<span size=\"small\"> ".$PkgList."</span>\n" ;
	}
	
	if ($Npkgs == 0) { $Npkgs = 1 }
	
	$ProgressBar->set_fraction(0);
	my $Range = 0.85 / $Npkgs  ;
	my $Progress = 0.05;
	$ProgressBar->set_fraction($Progress);
	Gtk2->main_iteration while ( Gtk2->events_pending );
	
	for $soft ( keys %db ) {
		if ( defined @{$db{$soft}}[INSTALLED_PKG] ) { 
			if ( @{$db{$soft}}[SELECTED_AS_PKG] ) {
			
				$ProgressBar->set_text("Removing @{$db{$soft}}[INSTALLED_PKG]");
				$Netpkg->Remove($soft);
				
				# %db cleanup
				if ( defined @{$db{$soft}}[AVAILABLE_PKG]) {
					@{$db{$soft}}[STATUS] = STATUS_NEW ;
					@{$db{$soft}}[SELECTED_AS_PKG] = 0 ;					
				}else{
					delete($db{$soft});
				}
				
			$Progress = $Progress + $Range ;
			$ProgressBar->set_fraction($Progress);
			Gtk2->main_iteration while ( Gtk2->events_pending );
			}
		}
	}	
	
	$ProgressBar->set_text("Refreshing database");			
	$ProgressBar->set_fraction(0.9);
	Gtk2->main_iteration while ( Gtk2->events_pending );
	
	&BuildLocalDB();
				
	$ProgressBar->set_fraction(0.95);	
	Gtk2->main_iteration while ( Gtk2->events_pending );
	$TreeStore->clear;
	&FillTree();
	
	$ProgressBar->set_fraction(1);
	$ProgressBar->set_text("Done");
	Gtk2->main_iteration while ( ! Gtk2->events_pending );	
	$ProgressBar->set_fraction(0);
	$ProgressBar->set_text("");
	$InfoLabel->set_text("");
	$LaunchButton->set_sensitive( TRUE );
	$ScrolledWindow->set_sensitive( TRUE );
	$RefreshButton->set_sensitive( TRUE );
	
	# Configuring the window
	#$window2->signal_connect(destroy => \&ActionExit);
	$window2->set_default_size (500, 150);
	$window2->set_resizable(TRUE);
	$window2->set_title("Report");
	$window2->set_border_width(7);
	$window2->set_icon_from_file("$WindowIconPath");
	
	
	# info section
	my $InfoLabel = Gtk2::Label->new;
	$InfoLabel->set_width_chars(100);
	$InfoLabel->set_justify('GTK_JUSTIFY_FILL');
	$InfoLabel->set_line_wrap (TRUE);
	$InfoLabel->set_line_wrap_mode ('word');

	my $InfoBox = Gtk2::HBox->new(TRUE,0);
	$InfoBox->set_border_width(7);
	$InfoBox->pack_start($InfoLabel, TRUE, TRUE, 0);

	my $InfoFrame = Gtk2::Frame->new('Uninstall process result');
	$InfoFrame->set_border_width(3);
	$InfoFrame->add($InfoBox);	
	
	$InfoLabel->set_markup($Markup);

	# Actions ##################################"
	my $ExitButton = Gtk2::Button->new_from_stock('gtk-close');
	$ExitButton->signal_connect(clicked => sub{ $window2->destroy; } ) ; 
			
	my $ActionBox = Gtk2::HBox->new;		
	$ActionBox->pack_end($ExitButton, FALSE, FALSE, 0);

	my $OuterBox = Gtk2::VBox->new;
	$OuterBox->pack_start($InfoFrame, TRUE, TRUE, 0);
	$OuterBox->pack_start($ActionBox, FALSE, FALSE, 0);
	$window2->add($OuterBox);		
	$window2->show_all;
	
	return TRUE ;		
}


